#!/usr/bin/python
#
# ecryptfs-mount-helper-gui.py - graphical ecryptfs mount helper
# Copyright (c) 2008 Red Hat, Inc.
# Author(s): Jarod Wilson <jarod@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk
import gobject
import gtk.glade

import sys
import os

##
## I18N
##
from rhpl.translate import _, N_
import rhpl.translate as translate
domain = "ecryptfs-mount-helper-gui"
translate.textdomain (domain)
gtk.glade.bindtextdomain(domain)

if os.access("ecryptfs-mount-helper-gui.glade", os.F_OK):
    xml = gtk.glade.XML ("./ecryptfs-mount-helper-gui.glade", domain=domain)
else:
    xml = gtk.glade.XML ("/usr/share/ecryptfs-utils/ecryptfs-mount-helper-gui.glade", domain=domain)

PATH_DEFAULT = "/mnt/ecryptfs"
MOUNT = "/bin/mount"
FS = "ecryptfs"
MINVER = 119
CIPHER_DEFAULT =  "aes"
KEYSIZE_DEFAULT = "16"

# ecryptfs features, extracted from include/ecryptfs.h, used to check
# whether or not we should expose a given setting (though at this time,
# only plaintext passthrough and hmac are actually relevant).
ECRYPTFS_VERSIONING_PASSPHRASE = 0x00000001
ECRYPTFS_VERSIONING_PUBKEY = 0x00000002
ECRYPTFS_VERSIONING_PLAINTEXT_PASSTHROUGH = 0x00000004
ECRYPTFS_VERSIONING_POLICY = 0x00000008
ECRYPTFS_VERSIONING_XATTR = 0x00000010
ECRYPTFS_VERSIONING_MULTKEY = 0x00000020
ECRYPTFS_VERSIONING_MISCDEV = 0x00000040
ECRYPTFS_VERSIONING_HMAC = 0x00000080
ECRYPTFS_VERSIONING_FILENAME_ENCRYPTION = 0x00000100
ECRYPTFS_VERSIONING_GCM = 0x00000200

debug = 0
testing = 0

"""
    TODO:
        * support openssl mounts
        * support passfile mounting
        * handle sig cache
"""

class mainWindow:
    def __init__(self):
        self.xml = xml
        self.mount = MOUNT
        self.path = PATH_DEFAULT
        self.backingstore = PATH_DEFAULT
        self.fs = FS
        self.cipher = CIPHER_DEFAULT
        self.keysize = KEYSIZE_DEFAULT
        self.customBacking = False
        self.advOpts = False
        self.passthrough = False
        self.hmac = True


    def setupScreen(self):
        # Main window setup
        self.toplevel = self.xml.get_widget("mainWindow")
        self.toplevel.set_position(gtk.WIN_POS_CENTER)

        # Menu items
        self.fileOK = self.xml.get_widget("ok_imagemenuitem")
        self.fileQuit = self.xml.get_widget("quit_imagemenuitem")
        self.helpAbout = self.xml.get_widget("about_imagemenuitem")

        # Dialog boxes
        self.aboutDialog = self.xml.get_widget("aboutdialog")
        self.successDialog = self.xml.get_widget("success_dialog")
        self.failureDialog = self.xml.get_widget("failure_dialog")

        # eCryptfs Mount Point
        self.mountPointChooser = self.xml.get_widget("mountpoint_chooser")
        self.mountPointChooser.set_current_folder("%s" % self.path)

        # Backing Store
        self.backingStoreRadio = self.xml.get_widget("backingstore_radio2")
        self.backingStoreChooser = self.xml.get_widget("backingstore_chooser")
        self.backingStoreChooser.set_current_folder("%s" % self.backingstore)

        # Passphrase entry
        self.passphraseEntry = self.xml.get_widget("passphrase_entry")

        # Advanced options
        self.advCheckButton = self.xml.get_widget("advanced_checkbutton")
        self.cipherLabel = self.xml.get_widget("ciphers_label")
        self.cipherCombo = self.xml.get_widget("ciphers_combobox")
        self.cipherCombo.set_active(0) # show aes greyed out
        self.keysizeLabel = self.xml.get_widget("keysize_label")
        self.keysizeCombo = self.xml.get_widget("keysize_combobox")
        self.keysizeCombo.set_active(0) # show 128-bit greyed out
        self.passthroughCheck = self.xml.get_widget("passthrough_checkbutton")
        self.hmacCheck = self.xml.get_widget("hmac_checkbutton")

        # Buttons
        self.okButton = self.xml.get_widget("ok_button")
        self.quitButton = self.xml.get_widget("quit_button")
        self.successQuitButton = self.xml.get_widget("success_quit_button")
        self.failureOKButton = self.xml.get_widget("failure_ok_button")
        self.failureQuitButton = self.xml.get_widget("failure_quit_button")


    def setupTooltips(self):
        self.tooltips = gtk.Tooltips()
        self.tooltips.set_tip(self.backingStoreRadio, "Normally, its best to "
                              "mount your eCryptfs overlay at the same "
                              "location as its backing store, so as to avoid "
                              "confusion, but this isn't a requirement...")
        self.tooltips.set_tip(self.advCheckButton, "The default values "
                              "should work for most users, but you can alter "
                              "a few things here, if you so desire. Note that "
                              "some options may not be available on all "
                              "kernel versions.")
        self.tooltips.set_tip(self.passthroughCheck, "Pass through unencrypted "
                              "data already on backing store")
        self.tooltips.set_tip(self.hmacCheck, "Use HMAC to ensure data "
                              "integrity and authenticity")


    def sanityCheck(self):
        # Make sure ecryptfs module is loaded
        if not os.access("/sys/fs/ecryptfs/version", os.F_OK):
            os.popen("/sbin/modprobe -q ecryptfs")
        if not os.access("/sys/fs/ecryptfs/version", os.F_OK):
            self.showErrorMessage(_("Your kernel doesn't appear to have any "
                                    "support for eCryptfs! Exiting..."))
            sys.exit(1)
        # We'll use this later to test for existing features (its a bitmap)
        self.ecryptfsVer = int(os.popen("cat /sys/fs/ecryptfs/version").read().strip())
        if debug:
            print "eCryptfs kernel version:", self.ecryptfsVer

    # Set up actions to take upon stuff getting clicked
    def setupActions(self):
        self.toplevel.connect("destroy", self.destroy)
        self.fileOK.connect("activate", self.tryMount)
        self.fileQuit.connect("activate", self.destroy)
        self.helpAbout.connect("activate", self.aboutClicked)

        self.mountPointChooser.connect("selection-changed", self.mountPointChanged)
        self.backingStoreRadio.connect("toggled", self.backingStoreRadioToggled)
        self.backingStoreChooser.connect("selection-changed", self.backingStoreChanged)

        self.advCheckButton.connect("toggled", self.advCheckButtonToggled)
        self.cipherCombo.connect("changed", self.setCipher)
        self.keysizeCombo.connect("changed", self.setKeysize)
        self.passthroughCheck.connect("toggled", self.passthroughToggled)
        self.hmacCheck.connect("toggled", self.hmacToggled)

        self.okButton.connect("clicked", self.tryMount)
        self.quitButton.connect("clicked", self.destroy)
        self.successQuitButton.connect("clicked", self.destroy)
        self.failureOKButton.connect("clicked", self.hideFailureDialog)
        self.failureQuitButton.connect("clicked", self.destroy)



    def tryMount(self, *args):
        # Check to see if ecryptfs is already mounted here
        path = self.mountPointChooser.get_current_folder()
        mntpts = open("/proc/mounts").readlines()
	for mntpt in mntpts:
	    if mntpt.find("ecryptfs") != -1:
                if mntpt.find(path) != -1:
	            self.showErrorMessage(_("An eCryptfs overlay is already "
                                            "mounted at %s, please choose "
                                            "another path." % path))
                    return 1
	# Figure out the mount options to use
        if self.customBacking:
            bs = self.backingStoreChooser.get_current_folder()
        else:
            bs = path
        passphrase = self.passphraseEntry.get_text()
        if not passphrase:
            self.showErrorMessage(_("Empty passphrases are not allowed! "
                                    "Please input a valid passphrase..."))
            return 1
        mountopts = "key=passphrase:passwd=%s" % passphrase
        if self.advOpts:
            cipher = self.cipher
            keysize = self.keysize
            mountopts += (",ecryptfs_cipher=%s,ecryptfs_key_bytes=%s" 
                          % (cipher, keysize))
            if self.passthrough:
                mountopts += ",ecryptfs_passthrough"
            if self.hmac and self.ecryptfsVer & ECRYPTFS_VERSIONING_HMAC:
                mountopts += ",ecryptfs_hmac"
        else:
            cipher = CIPHER_DEFAULT
            keysize = KEYSIZE_DEFAULT
            mountopts += (",ecryptfs_cipher=%s,ecryptfs_key_bytes=%s"
                          % (cipher, keysize))
            if self.ecryptfsVer & ECRYPTFS_VERSIONING_HMAC:
                mountopts += ",ecryptfs_hmac"
        mountopts += ",verbosity=0,no_sig_cache"

        # Now try actually mounting this beast...
        success = False
        err = os.popen("%s -t %s %s %s -o %s" % (MOUNT, FS, bs, path, mountopts)).readlines()
        if debug:
            print "eCryptfs mount string:"
            print "**** %s -t %s %s %s -o %s" % (MOUNT, FS, bs, path, mountopts)
            print " "
        for line in err:
            if debug:
                print line.strip("\n")
            if line.find("Mounted eCryptfs") != -1:
                success = True
        if success:
            # clicking the quit button in the dialog will get us out of here
            self.successDialog.show()
        else:
            self.failureDialog.show()

    def hideFailureDialog(self, *args):
        self.failureDialog.hide()

    def aboutClicked(self, *args):
        if debug:
            print "about menu item clicked"
        def close(w, res):
            if res == gtk.RESPONSE_CANCEL:
                w.hide()
        self.aboutDialog.connect("response", close)
        self.aboutDialog.show()

    def mountPointChanged(self, *args):
        self.path = self.mountPointChooser.get_current_folder()
        if debug:
            print "Path now", self.path
        if not self.customBacking:
            self.backingStoreChooser.set_current_folder(self.path)
            self.backingstore = self.path

    def backingStoreChanged(self, *args):
        self.backingstore = self.backingStoreChooser.get_current_folder()
        if debug:
            print "backing store now", self.backingstore

    def backingStoreRadioToggled(self, *args):
        self.customBacking = not self.customBacking
        self.backingStoreChooser.set_sensitive(self.customBacking)

    def advCheckButtonToggled(self, *args):
        self.advOpts = not self.advOpts
        if debug:
            print "setting advCheckButton to", self.advOpts

        self.cipherLabel.set_sensitive(self.advOpts)
        self.cipherCombo.set_sensitive(self.advOpts)
        self.keysizeLabel.set_sensitive(self.advOpts)
        self.keysizeCombo.set_sensitive(self.advOpts)
        if self.ecryptfsVer & ECRYPTFS_VERSIONING_PLAINTEXT_PASSTHROUGH:
            self.passthroughCheck.set_sensitive(self.advOpts)
        elif debug and self.advOpts:
            print "plaintext passthrough is not supported on this kernel"
        if self.ecryptfsVer & ECRYPTFS_VERSIONING_HMAC:
            self.hmacCheck.set_sensitive(self.advOpts)
        elif debug and self.advOpts:
            print "hmac is not supported on this kernel"
        if not self.advOpts:
            self.cipherCombo.set_active(0) # show aes greyed out
            self.keysizeCombo.set_active(0) # show 128-bit greyed out

    # kludgy setting of keysize options per cipher... :\
    def setAvailKeysizes(self, *args):
        if self.cipher == "des":
            self.keysizeCombo.prepend_text("64-bit")
        elif self.cipher == "des3_ede":
            self.keysizeCombo.prepend_text("192-bit")
        else:
            self.keysizeCombo.prepend_text("128-bit")
        rows = 5
        while rows > 0:
            self.keysizeCombo.remove_text(rows)
            rows = rows - 1
        if self.cipher == "aes":
            self.keysizeCombo.append_text("192-bit")
        bigkeys = [ 'aes', 'blowfish', 'twofish', 'cast6' ]
        for cipher in bigkeys:
            if cipher == self.cipher:
                self.keysizeCombo.append_text("256-bit")
        self.keysizeCombo.set_active(0)

    def setCipher(self, *args):
        self.cipher = self.cipherCombo.get_active_text()
        self.setAvailKeysizes()
        if debug:
            print "Setting cipher to", self.cipher

    def setKeysize(self, *args):
        keysizeText = self.keysizeCombo.get_active_text()
        self.keysize = int(keysizeText.split("-")[0]) / 8
        if debug:
            print "Setting keysize to", self.keysize

    def passthroughToggled(self, *args):
        self.passthrough = not self.passthrough
        if debug:
            print "Passthrough toggled to", self.passthrough

    def hmacToggled(self, *args):
        self.hmac = not self.hmac
        if debug:
            print "HMAC toggled to", self.hmac

    def run(self):
        self.toplevel.show_all()
        gtk.main()

    def destroy(self, *args):
        gtk.main_quit()

    def showErrorMessage(self, text):
        if debug:
            print >> sys.stderr, text

        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, 
                                gtk.BUTTONS_OK, text)
        dlg.set_position(gtk.WIN_POS_CENTER)
        dlg.set_modal(True)
        dlg.run()
        dlg.destroy()


if __name__ == "__main__":
    import getopt

    opt, arg = getopt.getopt(sys.argv[1:], 'dth', ['debug', 'test', '', 'help'])
    for (opt, val) in opt:
        if opt in ("-d", "--debug"):
            print "*** Debug messages enabled. ***"
            debug = 1
        elif opt in ("-t", "--test", "--testing"):
            print "*** Testing only. No configurations will be modified. ***"
            testing = 1
        elif opt in ("-h", "--help"):
            print >> sys.stderr, "Usage: ecryptfs-mount-helper-gui.py [--test] [--debug]"

    if not testing:
        uid = os.getuid()
        if uid != 0:
            w = mainWindow()
            w.showErrorMessage(_("You must be root to run this application."))
            sys.exit()

    win = mainWindow()
    win.setupScreen()
    win.setupTooltips()
    win.sanityCheck()
    win.setupActions()
    win.run()
